/*!
 * \file 
 * \brief UsbUart Click example
 * 
 * # Description
 * This example reads and processes data from USB UART clicks.
 *
 * The demo application is composed of two sections :
 * 
 * ## Application Init 
 * Initializes driver and power module.
 * 
 * ## Application Task  
 * Reads data and echos it back to device and logs it to board.
 * 
 * \author MikroE Team
 *
 */
// ------------------------------------------------------------------- INCLUDES

#include "board.h"
#include "log.h"
#include "usbuart.h"
#include "string.h"

#define PROCESS_RX_BUFFER_SIZE 500

// ------------------------------------------------------------------ VARIABLES

static usbuart_t usbuart;
static log_t logger;

static int32_t rsp_size;
static char uart_rx_buffer[ PROCESS_RX_BUFFER_SIZE ] = { 0 };

// ------------------------------------------------------ APPLICATION FUNCTIONS

void application_init ( void )
{
    log_cfg_t log_cfg;
    usbuart_cfg_t cfg;

    //  Logger initialization.

    LOG_MAP_USB_UART( log_cfg );
    log_cfg.level = LOG_LEVEL_DEBUG;
    log_cfg.baud = 115200;
    log_init( &logger, &log_cfg );
    log_info( &logger, "---- Application Init ----" );

    //  Click initialization.

    usbuart_cfg_setup( &cfg );
    USBUART_MAP_MIKROBUS( cfg, MIKROBUS_1 );
    usbuart_init( &usbuart, &cfg );

    usbuart_pwr_ctrl( &usbuart, USBUART_POWER_ON );
    usbuart_set_cts( &usbuart, USBUART_CTS_NO_ACTIVE );
    usbuart_set_mode( &usbuart, USBUART_MODE_NORMAL );
}

void application_task ( void )
{    
    rsp_size = usbuart_generic_read( &usbuart, uart_rx_buffer, PROCESS_RX_BUFFER_SIZE );

    if ( rsp_size > 0 )
    {  
        usbuart_generic_write( &usbuart, uart_rx_buffer, rsp_size );
        log_printf( &logger, "%s", uart_rx_buffer );
        memset( uart_rx_buffer, 0, rsp_size );
    }  
}

void main ( void )
{
    application_init( );

    for ( ; ; )
    {
        application_task( );
    }
}

// ------------------------------------------------------------------------ END
